@@ -5,13 +5,13 @@
 #  NOVELTIS: Cedric Bacour / Ivan Price
 ###############################################################
 
-#from __init__ import *
 
 import os
 import numpy as np
 import netCDF4
 import socket
 import logging
+import process_brdf
 
 
 class adam_config():
@@ -81,23 +81,23 @@ class adam_config():
 
 
 
-        self.netcdf_datafile = 'ADAM_V1_PlateCarree.nc'
-        self.netcdf_file_covarland_template = 'ADAM_V1_ERR_PlateCarree_M%02i.nc'
+        self.netcdf_datafile = 'ADAM_V2_PlateCarree.nc'
+        self.netcdf_file_covarland_template = 'ADAM_V2_ERR_PlateCarree_M%02i.nc'
         self.path_ascii = os.path.join(self.path_data, 'ascii')                     # path where the ascii files are saved
         self.path_maps = os.path.join(self.path_data,'maps_raw')                    # output path for saving maps
 
         # - Land
         # refractive index from http://refractiveindex.info/?group=CRYSTALS&material=H2O-ice,
         #  Warren S.G (1984, Optical constants of ice from the ultraviolet to the microwave, Applied Optics, 23:1206-1225)
-        self.file_land_IceRefInd = 'ADAM_V1_IceComplexRefractiveIndex_300-4000.txt'
+        self.file_land_IceRefInd = 'ADAM_IceComplexRefractiveIndex_300-4000.txt'
         # - Ocean
-        self.file_ocean_ref_std = 'ADAM_V1_OceanTypicalReflectance_300-800.txt'
+        self.file_ocean_ref_std = 'ADAM_OceanTypicalReflectance_300-800.txt'
         # refractive index from http://refractiveindex.info/?group=LIQUIDS&material=Water
         #  G. M. Hale and M. R. Querry. Optical Constants of Water in the 200-nm to 200-microm Wavelength Region, Appl. Opt. 12, 555-563 (1973) doi:10.1364/AO.12.000555
-        self.file_ocean_WaterRefInd = 'ADAM_V1_WaterRefractiveIndex_300-4000.txt'
+        self.file_ocean_WaterRefInd = 'ADAM_WaterRefractiveIndex_300-4000.txt'
         # absorption coefficient of water
         # from  D. J. Segelstein, "The complex refractive index of water," University of Missouri-Kansas City, (1981). (http://omlc.ogi.edu/spectra/water/data/segelstein81.dat)
-        self.file_ocean_WaterAbs = 'ADAM_V1_WaterAbsorption_300-4000.txt' # CHANGE_CB 
+        self.file_ocean_WaterAbs = 'ADAM_WaterAbsorption_300-4000.txt' # CHANGE_CB
 
 
 
@@ -108,7 +108,8 @@ class adam_config():
         self.netcdf_datafile = os.path.join(self.path_data, self.netcdf_datafile)
         self.netcdf_file_covarland_template = os.path.join(self.path_data, self.netcdf_file_covarland_template)
 
-        logging.basicConfig(filename=debug_log_file, level=logging.DEBUG, format='%(asctime)s %(levelname)s %(message)s')
+
+        logging.basicConfig(filename=self.debug_log_file, level=logging.DEBUG, format='%(asctime)s %(levelname)s %(message)s')
 
 
     # -
@@ -129,7 +130,7 @@ class adam_config():
 
 
         # - standard viewing geometry (in degrees)
-        self.sza_std = 40
+        self.sza_std = 45
         self.vza_std = 0
         self.phi_std = 0
 
@@ -140,6 +141,10 @@ class adam_config():
 
         self.step_polar = 41 # discretisation step for drawing polar plots
 
+        # standard BRDF F1 & F2 functions over land (in the standard observation geometry)
+        self.f1_std = process_brdf.brdf_land_f1(self.sza_std,self.vza_std,self.vza_std)
+        self.f2_std = process_brdf.brdf_land_f2(self.sza_std,self.vza_std,self.vza_std)
+
         # maximum value authorized
         self.sza_max_limit = 70
         self.vza_max_limit = 70
@@ -168,44 +173,32 @@ class adam_config():
         self.ilmbd_G = np.where(self.lmbd == self.lmbd_G)[0][0]
         self.ilmbd_B = np.where(self.lmbd == self.lmbd_B)[0][0]
 
+
         try:
             source_dataset = netCDF4.Dataset(self.netcdf_datafile, 'r')
         except:
             raise Exception('Error opening main source data file, location given was: %s' % self.netcdf_datafile)
 
+
         # load the modis wavebands for convenience later
         self.modis_wavebands = np.round(source_dataset.variables['wavebands'][:])
         source_dataset.close()
+        self.indices_modis_bands = []
+        for wl in self.modis_wavebands:
+            ind = np.ma.nonzero(self.lmbd == wl )[0]
+            self.indices_modis_bands.extend(ind.tolist())
+
 
         self.land , self.ocean = self._get_land_ocean_configs()
 
-    # - Land/Snow/Sea masks (INFO FOR IVAN : will be populated in process_reflectance)
-#    mask_land = None
-#    mask_snow = None
-#    mask_ocean = None
-#    idx_land = []
-#    idx_snow = []
-#    idx_ocean = []
-
 
     def _get_land_ocean_configs(self):
 
-
-        # - define Global class
-        #    cfg = defCglobal(limit_square_degrees, limit_degrees_pixel_calcs, vars_land,vars_ocean,
-        #                     sza_std,vza_std,phi_std,vza_range,vza_step,vza_values,phi_values,step_polar,
-        #                     missval_1b,missval_2b,spectral_domains,
-        #                     max_pts_analysis,
-        #                     lmbd,lmbd_R,lmbd_G,lmbd_B,ilmbd_R,ilmbd_G,ilmbd_B,
-        #                     path_data,netcdf_datafile,path_figures,path_maps,
-        #                     netcdf_file_covarland_template,
-        #                     OUTPUT_ROOT_DIR,DEBUG_LOG_FILE,HTTP_ROOT,
-        #                     mask_land,mask_snow,mask_ocean,idx_land,idx_snow,idx_ocean)
-
         # -
         # - Land
         # -
         file_acp =  os.path.join(self.path_data,'ADAM_V1_ACP.nc')
+
         try:
             dataACP = netCDF4.Dataset(file_acp, 'r')
         except:
@@ -213,15 +206,14 @@ class adam_config():
 
         kice_data = np.loadtxt(os.path.join(self.path_ascii, self.file_land_IceRefInd), comments='#', skiprows=0, unpack=True)
 
-        AttributeDict
         config_land = AttributeDict({
             'missval'             : -8421,
             'add_offset_land'     : 0.,
             'scale_factor_land'   : 0.0001,
 
             # - Spectrum parameters (ACP)
-            'file_acp'            : (os.path.join(self.path_data,'ADAM_V1_ACP.nc')),
-            'ACP_eigenvectors'    : dataACP.variables['Eigenvectors'][:],
+            'file_acp'            : file_acp,
+            #'ACP_eigenvectors'    : dataACP.variables['Eigenvectors'][:, dataACP.variables['neigenvectors'][:]],
             'ACP_ObsMean'         : dataACP.variables['ObsMean'][:],
             'ACP_SpecMean'        : dataACP.variables['SpecMean'][:],
             'ACP_prod'            : dataACP.variables['prodACP'][:],
@@ -240,8 +232,8 @@ class adam_config():
             'err_V'                   : 0.15,
 
             # - snow
-            'snow_size_grain'         : 0.12e-3,
-            'chanB'                   : 0, #CHANGE_CB 
+            'snow_grain_size'         : 0.12e-3,
+            'chanB'                   : 0, #CHANGE_CB
             'chanR'                   : 2,
             'chanPIR'                 : 3,
             'chanPIR2'                : 4,
@@ -254,10 +246,11 @@ class adam_config():
             'kice'                    : kice_data[1,:]
         })
 
-        #ACP_eigenvectors = ACP_eigenvectors[:,ACP_neigenvectors][:]
-        #ACP_mat = np.dot(ACP_eigenvectors,ACP_prod)
-        config_land['ACP_eigenvectors'] = config_land['ACP_eigenvectors'][:, config_land['ACP_neigenvectors']][:]
-        # can may be computed only once
+
+        # this consumes ~5 seconds !
+        config_land['ACP_eigenvectors'] = dataACP.variables['Eigenvectors'][:, config_land['ACP_neigenvectors']]
+        #config_land['ACP_eigenvectors'] = config_land['ACP_eigenvectors'][:, config_land['ACP_neigenvectors']][:]
+
         config_land['ACP_mat'] = np.dot(config_land['ACP_eigenvectors'], config_land['ACP_prod'])
 
         dataACP.close()
@@ -267,7 +260,6 @@ class adam_config():
     # -
     # - Ocean
     # -
-
         nwater_data = np.loadtxt(os.path.join(self.path_ascii, self.file_ocean_WaterRefInd), comments='#', skiprows=0, unpack=True)
 
         config_ocean = AttributeDict({
@@ -355,135 +347,3 @@ class AttributeDict(dict):
     def __setattr__(self, attr, value):
         self[attr] = value
 
-
-#
-## ===================================================================
-## land Class
-## ===================================================================
-#class defCland:
-#    """
-#    Land class containing all information required to compute
-#    reflectance spectra and BRDF for land pixels
-#    """
-#    ##@param[in] self   python implicit argument
-#    ##@param[in]
-#    ##@param[in]
-#    ##@param[in]
-#    ##@param[in]
-#    def __init__(self,slope_R,slope_V,intercept_R,intercept_V,alpha_RV,err_R,err_V,
-#                 missval,add_offset,scale_factor,
-#                 snow_size_grain,chanPIR2,chanPIR,chanR,cond_snow_1,cond_snow_2,
-#                 lmbd_kice,kice,
-#                 ACP_eigenvectors,ACP_ObsMean,ACP_SpecMean,ACP_prod,ACP_lmbd,ACP_neigenvectors,ACP_mat):
-#        self.slope_R         = slope_R
-#        self.slope_V         = slope_V
-#        self.intercept_R     = intercept_R
-#        self.intercept_V     = intercept_V
-#        self.alpha_RV        = alpha_RV
-#        self.err_R           = err_R
-#        self.err_V           = err_V
-#        self.missval         = missval
-#        self.add_offset      = add_offset
-#        self.scale_factor    = scale_factor
-#        self.lmbd_kice       = lmbd_kice
-#        self.kice            = kice
-#        self.snow_size_grain = snow_size_grain
-#        self.chanPIR2        = chanPIR2
-#        self.chanPIR         = chanPIR
-#        self.chanR           = chanR
-#        self.cond_snow_1     = cond_snow_1
-#        self.cond_snow_2     = cond_snow_2
-#        self.ACP_eigenvectors = ACP_eigenvectors
-#        self.ACP_ObsMean     = ACP_ObsMean
-#        self.ACP_SpecMean    = ACP_SpecMean
-#        self.ACP_prod        = ACP_prod
-#        self.ACP_lmbd        = ACP_lmbd
-#        self.ACP_neigenvectors = ACP_neigenvectors
-#        self.ACP_mat = ACP_mat
-#
-#
-## ===================================================================
-## ocean Class
-## ===================================================================
-#class defCocean:
-#    """
-#    Ocean class containing all information required to compute
-#    reflectance spectra and BRDF for ocean pixels
-#    """
-#    ##@param[in] self   python implicit argument
-#    ##@param[in]
-#    ##@param[in]
-#    ##@param[in]
-#    ##@param[in]
-#    def __init__(self,
-#                 missval,add_offset_ws,scale_factor_ws,add_offset_chl,scale_factor_chl,
-#                 lmbd_nwater,nwater,chl,ref_chl_std,lmbd_chl_std):
-#        self.missval          = missval
-#        self.add_offset_ws    = add_offset_ws
-#        self.scale_factor_ws  = scale_factor_ws
-#        self.add_offset_chl   = add_offset_chl
-#        self.scale_factor_chl = scale_factor_chl
-#        self.lmbd_nwater      = lmbd_nwater
-#        self.nwater           = nwater
-#        self.chl              = chl
-#        self.ref_chl_std      = ref_chl_std
-#        self.lmbd_chl_std     = lmbd_chl_std
-#
-#
-## ===================================================================
-## Global Class
-## ===================================================================
-#class defCglobal:
-#    """
-#    Gloabl class containing all information required to compute
-#    reflectance spectra and BRDF for land  & oceanpixels
-#    """
-#    def __init__(self,limit_square_degrees, limit_degrees_pixel_calcs, vars_land,vars_ocean,
-#                 sza_std,vza_std,phi_std,vza_range,vza_step,vza_values,phi_values,step_polar,
-#                 missval_1b,missval_2b,spectral_domains,
-#                 max_pts_analysis,
-#                 lmbd,lmbd_R,lmbd_G,lmbd_B,ilmbd_R,ilmbd_G,ilmbd_B,
-#                 path_data,netcdf_datafile,path_figures,path_maps,
-#                 netcdf_file_covarland_template,
-#                 OUTPUT_ROOT_DIR,DEBUG_LOG_FILE,HTTP_ROOT,
-#                 mask_land,mask_snow,mask_ocean,idx_land,idx_snow,idx_ocean):
-#
-#
-#        self.limit_square_degrees = limit_square_degrees
-#        self.limit_degrees_pixel_calcs = limit_degrees_pixel_calcs
-#        self.vars_land  = vars_land
-#        self.vars_ocean = vars_ocean
-#        self.sza_std = sza_std
-#        self.vza_std = vza_std
-#        self.phi_std = phi_std
-#        self.vza_range = vza_range
-#        self.vza_step = vza_step
-#        self.vza_values = vza_values
-#        self.phi_values = phi_values
-#        self.step_polar = step_polar
-#        self.missval_1b = missval_1b
-#        self.missval_2b = missval_2b
-#        self.spectral_domains = spectral_domains
-#        self.max_pts_analysis = max_pts_analysis
-#        self.lmbd = lmbd
-#        self.lmbd_R = lmbd_R
-#        self.lmbd_G = lmbd_G
-#        self.lmbd_B = lmbd_B
-#        self.ilmbd_R = ilmbd_R
-#        self.ilmbd_G = ilmbd_G
-#        self.ilmbd_B = ilmbd_B
-#        self.path_data = path_data
-#        self.netcdf_datafile = netcdf_datafile
-#        self.netcdf_file_covarland_template = netcdf_file_covarland_template
-#        self.path_figures = path_figures
-#        self.path_maps = path_maps
-#        self.OUTPUT_ROOT_DIR = OUTPUT_ROOT_DIR
-#        self.DEBUG_LOG_FILE = DEBUG_LOG_FILE
-#        self.HTTP_ROOT = HTTP_ROOT
-#        self.mask_land = mask_land
-#        self.mask_snow = mask_snow
-#        self.mask_ocean = mask_ocean
-#        self.idx_land = idx_land
-#        self.idx_snow = idx_snow
-#        self.idx_ocean = idx_ocean
-
