@@ -17,7 +17,7 @@ import adam
 
 
 
-def get_netcdf_data(extent, month, cfg):
+def get_netcdf_data(extent, month_index, cfg):
     ''' function to fetch netcdf4 data from the global netcdf file,
         according to a spatial extent window and the month index (0-11)
 
@@ -59,12 +59,13 @@ def get_netcdf_data(extent, month, cfg):
     if ind_min_y > ind_max_y :
         ind_min_y, ind_max_y = ind_max_y, ind_min_y
 
+
     # populate the variables according to the required range
     return_dataset = {}
     return_dataset['latitude']  = source_dataset.variables['latitude'][ind_min_y:ind_max_y]
     return_dataset['longitude'] = source_dataset.variables['longitude'][ind_min_x:ind_max_x]
 
-    return_dataset['month'] = [month]
+    return_dataset['month_index'] = month_index
 
     vars2read = cfg.vars_land + cfg.vars_ocean
     for varname in vars2read:
@@ -72,39 +73,35 @@ def get_netcdf_data(extent, month, cfg):
         # the vars with 3 dimensions (month, latitude, longitude, for vars:  chloro_conc and wind speed)
         if len(source_dataset.variables[varname].shape) == 3:
 
-            data = source_dataset.variables[varname][ind_min_x:ind_max_x,ind_min_y:ind_max_y,month]
-
+            data = source_dataset.variables[varname][ind_min_x:ind_max_x, ind_min_y:ind_max_y, month_index]
             idxNAN = np.ma.masked_equal(data, cfg.ocean.missval)
-
-            # unpack wind speed
-            #if varname == cfg.vars_ocean[0]:
-            #   data = data*Cocean.scale_factor_ws+Cocean.add_offset_chl
             # # unpack chlorophyll content
             if varname == cfg.vars_ocean[1]:
-                data = 10**(data) #*Cocean.scale_factor_chl+Cocean.add_offset_chl)
+                data = 10**(data)
 
             return_dataset[varname] = np.ma.filled(data,0)
 
         # a var with 4 dimensions (month,band,latitude,longitudefor var: ref_land)
         else:
-            data = source_dataset.variables[varname][ind_min_x:ind_max_x,ind_min_y:ind_max_y,:,month]
+            data = source_dataset.variables[varname][ind_min_x:ind_max_x, ind_min_y:ind_max_y, :, month_index]
             idxNAN = np.ma.masked_equal(data, cfg.land.missval)
-            # print data.shape
-            # print data[:,:,0]
-            #data = Cland.scale_factor*data+Cland.add_offset
             return_dataset[varname] = np.ma.filled(data,cfg.missval_2b)
 
     # close off the source NetCDF
     source_dataset.close()
 
-    # TODO: hack to set where land cover=255 (far ocean) to 0 (near ocean),
-    # this should probably be fixed in the source data
-    ###return_dataset['land_cover'] = np.ma.masked_equal(return_dataset['land_cover'], 255).filled(0)
 
+    # -- replace -9999 elements in ref_land by nan
+    idxNAN = np.ma.where(return_dataset['ref_land'] == cfg.missval_2b)
+    return_dataset['ref_land'][idxNAN] = np.nan
 
     # -- Error covariance matrix over land --
+    return_dataset['ref_land_covar'] = np.empty((ind_max_x-ind_min_x, ind_max_y-ind_min_y, 28))
+
+    
+    #for month_index in month_indexes:
     # open the source NetCDF read-only
-    error_data_filename = cfg.netcdf_file_covarland_template % (month+1)
+    error_data_filename = cfg.netcdf_file_covarland_template % (month_index+1)
     try:
         source_dataset = netCDF4.Dataset(error_data_filename, 'r')
     except:
@@ -112,13 +109,19 @@ def get_netcdf_data(extent, month, cfg):
 
     # read data
     data = source_dataset.variables['ref_land_covar'][ind_min_x:ind_max_x,ind_min_y:ind_max_y,:]
-    return_dataset['ref_land_covar'] = data
+    return_dataset['ref_land_covar'][:, :, :] = data
     source_dataset.close()
 
 
 
     # automatically calculate the land/water/snow masks
-    mask_land, mask_ocean, mask_snow, idx_land, idx_ocean, idx_snow = calculate_surface_masks(return_dataset['ref_land'], return_dataset['chloro_conc'], return_dataset['wind_speed'], cfg)
+    mask_land, mask_ocean, mask_snow, idx_land, idx_ocean, idx_snow = calculate_surface_masks(
+        return_dataset['ref_land'],
+        return_dataset['chloro_conc'],
+        return_dataset['wind_speed'],
+        cfg
+    )
+        
 
     return_dataset['mask_land']  = mask_land
     return_dataset['mask_ocean'] = mask_ocean
@@ -191,7 +194,8 @@ def calculate_surface_masks(ref_land, chloro_conc, wind_speed, cfg):
     # A pixel is declared as Ocean if wind_speed or chloro != 0 AND ref_land == NaN
 
     # mask out (the mask will be True) all pixels that are of the 'missing' value
-    mask_land_missing =  np.ma.masked_equal(ref_land[:,0], cfg.missval_2b)
+    #mask_land_missing =  np.ma.masked_equal(ref_land[:,0], cfg.missval_2b)
+    mask_land_missing =  np.ma.masked_invalid(ref_land[:,0])
 
     # some pixels in the land surface reflectance database are a value = 0
     # for each waveband => they can not be treated as land pixel nor ocean pixel => they
@@ -259,10 +263,10 @@ def calculate_surface_masks(ref_land, chloro_conc, wind_speed, cfg):
         #TODO: need commentry here about what this is doing !!!
         ##CHANGE_CB mask_snow_1 = np.ma.masked_greater(ref_land[idx_land,cfg.land.chanR], cfg.land.cond_snow_1)
         ##CHANGE_CB mask_snow_2 = np.ma.masked_less(ref_land[idx_land,cfg.land.chanPIR2]-ref_land[idx_land,cfg.land.chanR], cfg.land.cond_snow_2)
-        mask_snow_1 = np.ma.masked_greater(ref_land[idx_land,cfg.land.chanB], cfg.land.cond_snow_1) # CHANGE_CB 
-        mask_snow_2 = np.ma.masked_less(ref_land[idx_land,cfg.land.chanPIR2]-ref_land[idx_land,cfg.land.chanB], cfg.land.cond_snow_2) # CHANGE_CB 
+        mask_snow_1 = np.ma.masked_greater(ref_land[idx_land,cfg.land.chanB], cfg.land.cond_snow_1) # CHANGE_CB
+        mask_snow_2 = np.ma.masked_less(ref_land[idx_land,cfg.land.chanPIR2]-ref_land[idx_land,cfg.land.chanB], cfg.land.cond_snow_2) # CHANGE_CB
+
 
-        
         mask_snow = mask_snow_1.mask * mask_snow_2.mask
         idx_snow = np.ma.nonzero(mask_snow)
 
@@ -282,14 +286,6 @@ def calculate_surface_masks(ref_land, chloro_conc, wind_speed, cfg):
 
 
 
-    # set the config objects
-#    cfg.mask_land  = mask_land
-#    cfg.mask_ocean = mask_ocean
-#    cfg.mask_snow  = mask_snow
-#    cfg.idx_land   = idx_land
-#    cfg.idx_ocean  = idx_ocean
-#    cfg.idx_snow   = idx_snow
-
     return mask_land, mask_ocean, mask_snow, idx_land, idx_ocean, idx_snow
 
 
