@@ -7,15 +7,10 @@
 
 
 import numpy as np
+import process_reflectance
 
 
 
-#import adam
-
-
-#Cland  = cfg.land
-#Cocean = cfg.ocean
-
 
 # ===================================================================
 # compute: Compute the BRDF in the prescribed observation geometry
@@ -76,62 +71,23 @@ def main(reflectance, data, cfg, sza, lmbd, vza, phi,
             ans[1] = ref_err_ans
             return ans
         if beyond_za_limit == 'nan':
-            ans[0] = ref_ans*np.nan # Ivan / or cfg.missval ?
+            ans[0] = ref_ans*np.nan
             ans[1] = ref_err_ans
             return ans
         if beyond_za_limit == 'constant':
-            sza = np.clip(sza, -sza_max_limit, sza_max_limit) # Ivan / should it be done here or earlier when the angles are defined ?
+            sza = np.clip(sza, -sza_max_limit, sza_max_limit)
 
     if np.maximum.reduce(abs(vza)) > vza_max_limit:
         if beyond_za_limit == 'constant':
-            vza = np.clip(vza, -vza_max_limit, vza_max_limit) # Ivan / should it be done here or earlier when the angles are defined ?
-    # < MODIF_CB
+            vza = np.clip(vza, -vza_max_limit, vza_max_limit)
 
 
     # -
     # - Land processing
     # -
-    ##ref_err_ans = []
-
-
     if len(data['idx_land']) > 0:
         idx_land = data['idx_land']
 
-##        if multi_dir == True:
-##            ref_err_ans = np.zeros((len(data['idx_land']), nlmbd, vza.shape[0]), np.float32) + cfg.missval_2b # brdf error
-##        else:
-##            ref_err_ans = np.zeros((len(data['idx_land']), nlmbd), np.float32) + cfg.missval_2b # brdf error
-
-#        if not multi_dir:
-#            ipix = 0
-#            for idx in idx_land:
-##                print function_brdf_land(ref[idx,:],
-##                                                    ndvi[idx],
-##                                                    cfg,
-##                                                    sza = sza,
-##                                                    vza = vza,
-##                                                    phi = phi).shape
-#                ref_ans[idx,:] = function_brdf_land(ref[idx,:],
-#                                                    ndvi[idx],
-#                                                    cfg,
-#                                                    sza = sza,
-#                                                    vza = vza,
-#                                                    phi = phi)
-#
-#                if do_compute_error:
-#                    ref_err_ans[idx,:] = function_brdf_land_error(ref[idx,:],
-#                                                                   cfg,
-#                                                                   sza=sza,
-#                                                                   vza=vza,
-#                                                                   phi=phi,
-#                                                                   sza_std=cfg.sza_std,
-#                                                                   vza_std=cfg.vza_std,
-#                                                                   phi_std=cfg.phi_std )
-#
-#                ipix = ipix +1
-#
-#        # multidir = True
-#        else:
         ipix = 0
         for idx in idx_land:
             ans = function_brdf_land(ref[idx,:], ndvi[idx], cfg, sza=sza, vza=vza, phi=phi)
@@ -161,57 +117,32 @@ def main(reflectance, data, cfg, sza, lmbd, vza, phi,
 
     if len(data['idx_snow']) > 0:
 
-#        if multi_dir == False:
-#            for idx in data['idx_snow']:
-#                ref_ans[idx,:] = function_brdf_snow(cfg,
-#                                                    sza=sza,
-#                                                    vza=vza,
-#                                                    phi=phi,
-#                                                    lmbd=lmbd,
-#                                                    do_spectral_averaging=do_spectral_averaging,
-#                                                    multi_dir=False)
-#        elif multi_dir == True:
         for idx in data['idx_snow']:
             ans = function_brdf_snow( cfg,
                                       sza=sza,
                                       vza=vza,
                                       phi=phi,
                                       lmbd=lmbd,
-                                      do_spectral_averaging=do_spectral_averaging)
+                                      do_spectral_averaging=do_spectral_averaging,
+                                      data_snow = data['snow'][idx])
 
-            ref_ans[idx,:,:,:] = np.resize(ans,(1,nlmbd, vza.shape[0], phi.shape[0]))
+
+            ref_ans[idx,:,:,:] = np.resize(ans[0],(1,nlmbd, vza.shape[0], phi.shape[0]))
 
     # -
     # - Ocean processing
     # -
     if len(data['idx_ocean']) > 0:
-
-#        if multi_dir == False:
-#            for idx in data['idx_ocean']:
-#                ref_ans[idx,:] = function_brdf_ocean(ref[idx,:], ws[idx], cfg,
-#                                                     sza=sza, vza=vza, phi=phi,
-#                                                     lmbd=lmbd,
-#                                                     do_spectral_averaging=do_spectral_averaging,
-#                                                     multi_dir=False)
-#        elif multi_dir == True:
-
-        # > MODIF_CB
         # Foam component
         ans_foam = function_ocean_foam(cfg, ws[data['idx_ocean']], lmbd, do_spectral_averaging = do_spectral_averaging)
 
         # add contribution of foam
         ref_foam = np.zeros((len(data['idx_ocean']), nlmbd, len(vza), len(phi)))
-        ###########################
-        ###########################
-        # THIS HAS A BUG
+
         for ivza in range(len(vza)):
             for iphi in range(len(phi)):
-                #                print ans_foam.shape
-                #print ref_foam[:,:,ivza,iphi].shape
                 ref_foam[:,:,ivza,iphi] = ans_foam
 
-        # < MODIF_CB
-
         for iteration, idx in enumerate(data['idx_ocean']):
             ans = function_brdf_ocean(  ref[idx,:],
                                         ws[idx],
@@ -223,30 +154,17 @@ def main(reflectance, data, cfg, sza, lmbd, vza, phi,
                                         do_spectral_averaging=do_spectral_averaging,
                                         multi_dir=True)
 
-            # > MODIF_CB
-            #            ref_ans[idx,:,:] = np.resize(ans,(1, nlmbd, len(vza), len(phi))) / Ivan: I do not understand this syntax as the shape differs
             # dimension is pixel_no, wavelength, vza, phi
             # we need to perform a resize here as for numpy an array  of shape (17, 100)
             # is not the same as (1, 17, 100)
             ref_ans[idx,:,:,:] = np.resize(ans, (1, nlmbd, len(vza), len(phi))) + ref_foam[iteration,:,:,:]
-            # < MODIF_CB
 
 
-    # -
-    # - Reshape the reflectance matrix back to its original form
-    # -
-#    if multi_dir == False:
-#        ref_ans = np.reshape(ref_ans, (npix_X,npix_Y, nlmbd))
-#        ref_err_ans = np.reshape(ref_err_ans, (npix_X,npix_Y, nlmbd))
-#    else:
     ref_ans = np.reshape(ref_ans, (npix_X,npix_Y, nlmbd, len(vza), len(phi)))
     ref_err_ans = np.reshape(ref_err_ans, (npix_X,npix_Y, nlmbd, len(vza), len(phi)))
 
 
-    # > MODIF_CB
-    # -
     # - Eliminate out of range calculations with respect to view zenith angle
-    # -
     if beyond_za_limit in ['0','nan']:
         mask_vza = np.ma.masked_greater(abs(vza), vza_max_limit)
         idxBAD = np.ma.nonzero(mask_vza.mask == True)
@@ -257,11 +175,7 @@ def main(reflectance, data, cfg, sza, lmbd, vza, phi,
             ref_ans[:,:,idxBAD,:] = np.nan # IVAN / or cfg.missval ???
             ref_err_ans[:,:,idxBAD,:] = np.nan
 
-    # < MODIF_CB
 
-    # -
-    # - Return
-    # -
     ans = [[],[]]
     ans[0] = ref_ans
     ans[1] = ref_err_ans
@@ -300,20 +214,17 @@ def function_brdf_land(ref, ndvi, cfg, sza=None, vza=None, phi=None):
     if phi is None: phi = cfg.phi_std
 
     # syncronise shapes, this is neccessary because the shape of vza and phi need to always be the same
-    vza_meshed, phi_meshed = vza, phi
-    if vza.shape != phi.shape:
-        #vza_meshed, phi_meshed = np.meshgrid(vza, phi)
-        phi_meshed, vza_meshed = np.meshgrid(phi, vza) # CHANGE_CB
-        vza_meshed = vza_meshed.ravel()                # CHANGE_CB
-        phi_meshed = phi_meshed.ravel()                # CHANGE_CB
+    #vza_meshed, phi_meshed = vza, phi
+    #if vza.shape != phi.shape:
+    phi_meshed, vza_meshed = np.meshgrid(phi, vza)
+    vza_meshed = vza_meshed.ravel()               
+    phi_meshed = phi_meshed.ravel()               
 
 
     # BRDF fonctions
-    f1 = brdf_land_f1(sza,vza_meshed,phi_meshed) #.ravel()                # CHANGE_CB
-    f2 = brdf_land_f2(sza,vza_meshed,phi_meshed) #.ravel()                # CHANGE_CB
+    f1 = brdf_land_f1(sza,vza_meshed,phi_meshed)
+    f2 = brdf_land_f2(sza,vza_meshed,phi_meshed)
 
-    # print 'f1 shape: %s' % [f1.shape]
-    # print 'f2 shape: %s' % [f2.shape]
 
     # Scaling of the R and V coefficients as a function of NDVI
     #ndvi = (ref_MODIS[Cland.chanPIR]-ref_MODIS[Cland.chanR]) / (ref_MODIS[Cland.chanPIR]+ref_MODIS[Cland.chanR])
@@ -325,21 +236,25 @@ def function_brdf_land(ref, ndvi, cfg, sza=None, vza=None, phi=None):
     R = cfg.land.alpha_RV * Rnir + (1-cfg.land.alpha_RV)*Rr
     V = cfg.land.alpha_RV * Vnir + (1-cfg.land.alpha_RV)*Vr
 
-    # BRDF, # why would this ever happen ?
-    if f1.shape == () or ref.shape == ():
-        brdf = ref*(1+R*f1+V*f2)
+    # is it possible to provide commentry as to when each condition
+    # would be the case ? and the same for the snow and ocean ?
+    if f1.shape == () or ref.shape == (): # is this to determine if it is a float and not
+                                          # an array ? when would that happen ? why not
+                                          # use == type(float) ?
+        brdf = ref*(1+R*f1+V*f2)/(1+R*cfg.f1_std+V*cfg.f2_std)
     else:
         brdf = np.zeros( (ref.shape[0], len(vza), len(phi) ),np.float32)
         if ref.shape[0] > f1.shape[0]:
             for i in range(f1.shape[0]):
-                calc = ref[:]*(1+R*f1[i]+V*f2[i])
+                calc = ref[:]*(1+R*f1[i]+V*f2[i])/((1+R*cfg.f1_std+V*cfg.f2_std))
                 brdf[:,i] = calc.reshape(brdf[:,i].shape)
         else:
-            # for each waveband
+            # for each waveband (?)
             for i in range(ref.shape[0]):
-                calc = ref[i]*(1+R*f1[:]+V*f2[:])
+                calc = ref[i]*(1+R*f1[:]+V*f2[:])/(1+R*cfg.f1_std+V*cfg.f2_std)
                 brdf[i,:] = calc.reshape(brdf[i,:].shape)
 
+
     return brdf
 # END function_brdf_land
 # ===================================================================
@@ -373,31 +288,23 @@ def function_brdf_land_error(ref, cfg, sza=None, vza=None, phi=None,
     if vza_std is None: vza_std = cfg.vza_std
     if phi_std is None: phi_std = cfg.phi_std
 
-    # syncronise shapes, this is neccessary because the shape of vza and phi need to always be the same
-    vza_meshed, phi_meshed = vza, phi
-    if vza.shape != phi.shape:
-        #        vza_meshed, phi_meshed = np.meshgrid(vza, phi)
-        phi_meshed, vza_meshed = np.meshgrid(phi, vza) # CHANGE_CB
-        vza_meshed = vza_meshed.ravel()                # CHANGE_CB
-        phi_meshed = phi_meshed.ravel()                # CHANGE_CB
-
+    # syncronise shapes, this is neccessary because the shape of vza and phi need
+    # to always be the same
+    #vza_meshed, phi_meshed = vza, phi
+    #if vza.shape != phi.shape:
+    phi_meshed, vza_meshed = np.meshgrid(phi, vza)
+    vza_meshed = vza_meshed.ravel()
+    phi_meshed = phi_meshed.ravel()
 
     # BRDF fonctions
-    f1_std = brdf_land_f1(sza_std,vza_std,phi_std)
-    f2_std = brdf_land_f2(sza,vza_meshed,phi_meshed)
-
     f1 = brdf_land_f1(sza,vza_meshed,phi_meshed)
     f2 = brdf_land_f2(sza,vza_meshed,phi_meshed)
 
-
-    #nang = f1.shape[0] if f1.shape else 1
-    #err_brdf = np.zeros([ref.shape[0],nang],np.float32)
-
     err_brdf = np.zeros([ref.shape[0], len(vza), len(phi)],np.float32)
 
     # computation for each waveband
     for i in range(ref.shape[0]):
-        var_brdf = ref[i]**2 * (cfg.land.err_R**2*(f1-f1_std)**2 + cfg.land.err_V**2*(f2-f2_std)**2)
+        var_brdf = ref[i]**2 * (cfg.land.err_R**2*(f1-cfg.f1_std)**2 + cfg.land.err_V**2*(f2-cfg.f2_std)**2)
         calc = np.sqrt(var_brdf)
         err_brdf[i,:] = calc.reshape(err_brdf[i,:].shape)
 
@@ -417,7 +324,8 @@ def function_brdf_land_error(ref, cfg, sza=None, vza=None, phi=None,
 #
 def function_brdf_ocean(ref, wind_speed, cfg, sza=None, vza=None, phi=None, lmbd = None,
                         multi_dir = False, do_spectral_averaging = None):
-    '''Compute BRDF over Oean on a pixel basis from the Cox and Munk model
+    '''
+    Compute BRDF over Oean on a pixel basis from the Cox and Munk model
     See Breon FM (1993), An analytical model for the Cloud-Free Atmosphere/Ocean
     system reflectance, Remote Sensing of Environment, Remote Sensing of Environment,
     43:179-192
@@ -428,19 +336,19 @@ def function_brdf_ocean(ref, wind_speed, cfg, sza=None, vza=None, phi=None, lmbd
     Inputs
         - spectral reflectance
         - wind_speed [npixels]
-    - sun zenith angle       :  scalar
-    - view zenith angle      : scalar or vector
-    - relative azimuth angle : scalar or vector
-    - wavelength             : scalar or vector
-    - wavelength for nwater  : vector
-    - water refraction index : vector
-    - multi_dir              : logical
-
+        - sun zenith angle       :  scalar
+        - view zenith angle      : scalar or vector
+        - relative azimuth angle : scalar or vector
+        - wavelength             : scalar or vector
+        - wavelength for nwater  : vector
+        - water refraction index : vector
+        - multi_dir              : logical
 
 
     Outputs
         - reflectance [nlmbd, nvza]
 
+
     '''
 
     if sza is None: sza = cfg.sza_std
@@ -448,12 +356,11 @@ def function_brdf_ocean(ref, wind_speed, cfg, sza=None, vza=None, phi=None, lmbd
     if phi is None: phi = cfg.phi_std
 
     # syncronise shapes, this is neccessary because the shape of vza and phi need to always be the same
-    vza_meshed, phi_meshed = vza, phi
-    if vza.shape != phi.shape:
-        #        vza_meshed, phi_meshed = np.meshgrid(vza, phi)
-        phi_meshed, vza_meshed = np.meshgrid(phi, vza) # CHANGE_CB
-        vza_meshed = vza_meshed.ravel()
-        phi_meshed = phi_meshed.ravel()
+    #vza_meshed, phi_meshed = vza, phi
+    #if vza.shape != phi.shape:
+    phi_meshed, vza_meshed = np.meshgrid(phi, vza)
+    vza_meshed = vza_meshed.ravel()
+    phi_meshed = phi_meshed.ravel()
 
     dr = np.pi/180.
 
@@ -472,17 +379,6 @@ def function_brdf_ocean(ref, wind_speed, cfg, sza=None, vza=None, phi=None, lmbd
         nwater = cfg.ocean.nwater[idxWave]
 
 
-    # if nlmbd != 3701:
-    #     if do_spectral_averaging == True:
-    #         mask_lmbd = np.ma.masked_inside(cfg.ocean.lmbd_nwater,lmbd[0],lmbd[-1])
-    #         idxWave = np.ma.nonzero(mask_lmbd.mask == True)
-    #     else:
-    #         idxWave = []
-    #         for wl in lmbd:
-    #             ind = np.ma.nonzero(cfg.ocean.lmbd_nwater == wl)[0]
-    #             idxWave.extend(ind.tolist())
-    #     nwater = cfg.ocean.nwater[idxWave]
-
 
     # incidence angle = phase angle / 2
     mus = np.cos(sza*dr)
@@ -537,20 +433,23 @@ def function_brdf_ocean(ref, wind_speed, cfg, sza=None, vza=None, phi=None, lmbd
 # Compute reflectance over Ocean due to foam
 #
 def function_ocean_foam(cfg, ws, lmbd, do_spectral_averaging=False):
-    '''Compute the foam component of ocean reflectance based on
-    - Koepke P. (1984), Effective reflectance of oceanic whitecaps, Applied Optics, 23:11, 1816-1824.
-      to determine the contribution of the foam reflectance as a function of wind speed
-    - Kokhanovsky A.A. (2004), Spectral reflectance of whitecaps, Journal of Geophysical Research, 109, C05021,
-      doi:10.1029/2003JC002177
-      to determine its spectral dependancy
+    '''
+    Compute the foam component of ocean reflectance based on
+        - Koepke P. (1984), Effective reflectance of oceanic whitecaps, Applied Optics, 23:11, 1816-1824.
+          to determine the contribution of the foam reflectance as a function of wind speed
+        - Kokhanovsky A.A. (2004), Spectral reflectance of whitecaps, Journal of Geophysical Research, 109, C05021,
+          doi:10.1029/2003JC002177
+          to determine its spectral dependancy
 
 
     The foam reflectance is assumed isotropic
 
     Inputs
         - wind_speed [npixels]
+
     Outputs
         - reflectance [npixels]
+
     '''
 
     # determine the spectral dimension
@@ -570,18 +469,6 @@ def function_ocean_foam(cfg, ws, lmbd, do_spectral_averaging=False):
         foam_specmod = np.average(foam_specmod)
         nlmbd = 1
 
-    # if len(lmbd) != 3701:
-    #     if do_spectral_averaging == True:
-    #         mask_lmbd = np.ma.masked_inside(cfg.ocean.lmbd_nwater,lmbd[0],lmbd[-1])
-    #         idxWave = np.ma.nonzero(mask_lmbd.mask == True)
-    #         foam_specmod = np.average(foam_specmod[idxWave])
-    #         nlmbd = 1
-    #     else:
-    #         idxWave = []
-    #         for wl in lmbd:
-    #             ind = np.ma.nonzero(cfg.ocean.lmbd_nwater == wl)[0]
-    #             idxWave.extend(ind.tolist())
-    #         foam_specmod = cfg.ocean.foam_specmod[idxWave]
 
     # Contribution (from Koepke)
     contrib_foam = np.zeros(len(ws))
@@ -609,17 +496,19 @@ def function_ocean_foam(cfg, ws, lmbd, do_spectral_averaging=False):
 # Compute BRDF over Snow on a pixel basis
 #
 def function_brdf_snow( cfg, sza=None, vza=None, phi=None, lmbd=None,
-                        do_spectral_averaging=False):
-    '''Model of A.Kokhanovsky & E.P.Zege (2004, Scattering of snow, Applied
+                        do_spectral_averaging=False,
+                        fit2obs = False,  ref_obs = None, data_snow = None):
+    '''
+    Model of A.Kokhanovsky & E.P.Zege (2004, Scattering of snow, Applied
     Optics, 43(7):1589-1602)
 
     Inputs
-    - sun zenith angle (optional):       scalar
-    - view zenith angle (optional):      scalar or vector
-    - relative azimuth angle (optional): scalar or vector
-    - wavelength             : scalar or vector
-    - wavelength for kice  : vector
-    - imaginary part of ice refraction index : vector
+        - sun zenith angle (optional):       scalar
+        - view zenith angle (optional):      scalar or vector
+        - relative azimuth angle (optional): scalar or vector
+        - wavelength             : scalar or vector
+        - wavelength for kice  : vector
+        - imaginary part of ice refraction index : vector
 
 
     Outputs
@@ -637,89 +526,205 @@ def function_brdf_snow( cfg, sza=None, vza=None, phi=None, lmbd=None,
     if type(phi) != type(np.zeros(1)): phi=np.array([phi])
 
 
-    # syncronise shapes, this is neccessary because the shape of vza and phi need to always be the same
-    vza_meshed, phi_meshed = vza, phi
-    sza_meshed = sza
-    if vza.shape != phi.shape:
-        #        vza_meshed, phi_meshed = np.meshgrid(vza, phi)
-        phi_meshed, vza_meshed = np.meshgrid(phi, vza) # CHANGE_CB
-        vza_meshed = vza_meshed.ravel()
-        phi_meshed = phi_meshed.ravel()
-        sza_meshed = np.ones(len(vza_meshed))*sza_meshed
+    # - Synchronise shapes, this is neccessary because the shape of vza and phi need to always be the same
+    #vza_meshed, phi_meshed = vza, phi
+    #sza_meshed = sza
+    #if vza.shape != phi.shape:
+    phi_meshed, vza_meshed = np.meshgrid(phi, vza)
+    vza_meshed = vza_meshed.ravel()
+    phi_meshed = phi_meshed.ravel()
+    sza_meshed = np.ones(len(vza_meshed))*sza
 
     dr = np.pi/180
 
-    # Imaginary part of the ice refractive index in the required wavelength
-    nlmbd = len(lmbd)
-    kice = cfg.land.kice
 
-    # spectral subsampling
-    if nlmbd != len(cfg.lmbd):
-        idxWave = []
-        for wl in lmbd:
-            ind = np.ma.nonzero(cfg.land.lmbd_kice == wl)[0]
-            idxWave.extend(ind.tolist())
-        kice = kice[idxWave]
+    # -
+    # - if fit2obs == True, then the model is "fitted" to MODIS observations
+    # -
+        #  Principles:
+        #    1. Determine the fraction of non-snow cover
+        #    2. Determine the reflectance associated to non-snow (assuming a linear composition of the 2 components)
+    # if fveg > 0 (we can not neglet non snow elements)
+        #    3. Apply spectral interpolation procedure to non-snow reflectance: rveg
+        #    4. Recompute the pseudo-observation of snow reflectance as a function of MODIS observations and previous rveg
+        #    5. Fit the snow model at 1240 nm by estimating the snow grain size
+        #    6. Repeat 3
+        #    7. Repeat 4
+        #    8. Determine the total reflectance
+        # if fveg < 0.2
+        #    9. Fit the model to the observation at 1240 nm by estimating the snow grain size
+
+    if fit2obs == True:
+
+        data_snow = {} # structure containing the snow information for each pixel
+
+        # - Model: computed over all wavebands
+        RefSnow  = snow_model(cfg, sza_meshed, vza_meshed, phi_meshed, cfg.lmbd, cfg.land.kice)
+        rsnow = RefSnow.ravel()
+
+        # 1
+        fveg = np.clip((1-ref_obs[2])/0.95,0,1)
+        if fveg > 0: # mixed pixel
+
+            # iterative process of for estimating snow grain size and reflectance of non snow elements
+            for istep in range(2):
+                # 2 - 6
+                rveg_obs = (ref_obs - (1-fveg)*rsnow[cfg.indices_modis_bands].ravel())/fveg
+                # 3 - 7
+                rveg = process_reflectance.reflectance_spectrum_land(rveg_obs, cfg.lmbd, cfg)
+
+                if istep ==0:
+                    # 4
+                    rsnow_obs_new = (ref_obs-fveg*rveg[cfg.indices_modis_bands])/(1-fveg)
+                    snow_grain_size = snow_model(cfg, sza_meshed, vza_meshed, phi_meshed, cfg.lmbd, cfg.land.kice,
+                                                 invert_snow_grain_size = True,
+                                                 invert_ref_obs = rsnow_obs_new[4],
+                                                 invert_ilmbd = cfg.indices_modis_bands[4])
+                    # 5
+                    rsnow = snow_model(cfg, sza_meshed, vza_meshed, phi_meshed, cfg.lmbd, cfg.land.kice, snow_grain_size = snow_grain_size).ravel()
+
+            # 8
+            ans = fveg*rveg+(1-fveg)*rsnow
+            ans = ans.reshape((len(cfg.lmbd),len(vza),len(phi)))
+
+         # 9
+        if fveg <0.2:      # only snow
+            snow_grain_size = snow_model(cfg, sza_meshed, vza_meshed, phi_meshed, cfg.lmbd, cfg.land.kice,
+                                         invert_snow_grain_size = True,
+                                         invert_ref_obs = ref_obs[4],
+                                         invert_ilmbd = cfg.indices_modis_bands[4])
+            rsnow  = snow_model(cfg, sza_meshed, vza_meshed, phi_meshed, cfg.lmbd, cfg.land.kice, snow_grain_size = snow_grain_size).ravel()
+            rsnow = rsnow.reshape((len(cfg.lmbd),len(vza),len(phi)))
+            ans = rsnow.ravel()
+            rveg = rsnow[:]*0
+
+        data_snow['fveg'] = fveg
+        data_snow['snow_grain_size'] = snow_grain_size
+        data_snow['rveg'] = rveg
+        data_snow['rtot'] = ans.ravel()
+
+
+    # -
+    # - Compute the BRDF
+    # -
+        # if fveg > 0.2: mixed pixel considered as isotropic
+        # only snow otherwise
+    else:
+        if data_snow['fveg'] < 0.2: # only snow
+            RefSnow  = snow_model(cfg, sza_meshed, vza_meshed, phi_meshed, cfg.lmbd, cfg.land.kice, snow_grain_size = data_snow['snow_grain_size'] )*(1-data_snow['fveg'])
+            RefSnow = RefSnow.reshape((len(cfg.lmbd),len(vza),len(phi)))
+            ans = RefSnow
+
+            # - Spectral subsampling
+            if len(lmbd) != len(cfg.lmbd):
+                idxWave = []
+                for wl in lmbd:
+                    ind = np.ma.nonzero(cfg.land.lmbd_kice == wl)[0]
+                    idxWave.extend(ind.tolist())
+                ans = RefSnow[idxWave,:,:]
+
+        else: # mixed pixel: isotropic
+            RefSnow = np.zeros((len(lmbd),len(vza),len(phi)))
+            for iwl in range(len(lmbd)): RefSnow[iwl,:,:] = data_snow['rtot'][iwl]
+            ans = RefSnow
+
+
+        # - Spectral averaging
+        # if do_spectral_averaging:
+        #     ans  = np.zeros((1,len(vza_meshed)), np.float)
+        #     RefSnow = RefSnow.reshape(nlmbd,len(vza_meshed))
+        #     for i in range(len(vza)):  ans[:,i] = np.average(RefSnow[:,i])
+        #     ans = ans.reshape(1,len(vza),len(phi))
+
+    # -
+    # - return value
+    # -
+    return  [ans, data_snow]
+
+# END function_brdf_snow
+# ===================================================================
+
+
+
+# ===================================================================
+# Compute reflectance (spectro-directinal variations) over Snow
+#
+def snow_model( cfg, sza, vza, phi, lmbd, kice,
+                snow_grain_size = None,
+                invert_snow_grain_size = False, # needed to estimate snow grain size
+                invert_ref_obs         = None,          # needed to estimate snow grain size
+                invert_ilmbd           = None           # needed to estimate snow grain size
+                ):
+
+    '''Model of A.Kokhanovsky & E.P.Zege (2004, Scattering of snow, Applied
+    Optics, 43(7):1589-1602)
+
+    Inputs
+    - sun zenith angle:       scalar
+    - view zenith angle:      scalar or vector
+    - relative azimuth angle: scalar or vector
+    - wavelength             : scalar or vector
+    - wavelength for kice  : vector
+    - imaginary part of ice refraction index : vector
+
+
+    Outputs
+        - reflectance [nlmbd, nvza]
+
+    '''
+
+    if snow_grain_size == None: snow_grain_size = cfg.land.snow_grain_size
+
+    nlmbd = len(lmbd)
+    dr = np.pi/180
 
     # scattering angle
-    ksi = np.arccos(-np.cos(sza_meshed*dr)*np.cos(vza_meshed*dr)-np.sin(sza_meshed*dr)*np.sin(vza_meshed*dr)*np.cos(phi_meshed*dr)) * 180./np.pi
+    ksi = np.arccos(-np.cos(sza*dr)*np.cos(vza*dr)-np.sin(sza*dr)*np.sin(vza*dr)*np.cos(phi*dr)) * 180./np.pi
     # phase function
     PF = (11.1*np.exp(-0.087*ksi)+1.1*np.exp(-0.014*ksi))
     # value of R at zero absorption
     A = 1.247
     B = 1.186
     C = 5.157
-    Ref0 = (A+B*(np.cos(sza_meshed*dr)+np.cos(vza_meshed*dr))+C*np.cos(sza_meshed*dr)*np.cos(vza_meshed*dr)+PF)/(4*(np.cos(sza_meshed*dr)+np.cos(vza_meshed*dr)))
-    Ks = 3./7.*(1+2*np.cos(sza_meshed*dr))
-    Kv = 3./7.*(1+2*np.cos(vza_meshed*dr))
+    Ref0 = (A+B*(np.cos(sza*dr)+np.cos(vza*dr))+C*np.cos(sza*dr)*np.cos(vza*dr)+PF)/(4*(np.cos(sza*dr)+np.cos(vza*dr)))
+    Ks = 3./7.*(1+2*np.cos(sza*dr))
+    Kv = 3./7.*(1+2*np.cos(vza*dr))
 
-    alpha =  np.sqrt(4.*np.pi* kice *(13*cfg.land.snow_size_grain)/(lmbd*1e-9))
-
-    # spectral subsampling
-    if do_spectral_averaging:
-        alpha = np.array([np.average(alpha)])
-        nlmbd = 1
-
-    # spectral averaging or subsampling
-    # if do_spectral_averaging:
-    #     mask_lmbd = np.ma.masked_inside(cfg.land.lmbd_kice,lmbd[0],lmbd[-1])
-    #     idxWave = np.ma.nonzero(mask_lmbd.mask == True)
-    #     alpha = np.array([np.average(alpha[idxWave])])
-    #     nlmbd = 1
-    # else:
-    #     if nlmbd != len(cfg.lmbd):
-    #         idxWave = []
-    #         for wl in lmbd:
-    #             ind = np.ma.nonzero(cfg.land.lmbd_kice == wl)[0]
-    #             idxWave.extend(ind.tolist())
-    #         alpha = np.array(alpha[idxWave])
+    alpha =  np.sqrt(4.*np.pi* kice *(13*snow_grain_size)/(lmbd*1e-9))
 
 
-    #if multi_dir:
-    RefS  = np.zeros((nlmbd,len(vza_meshed)), np.float)
+    # - Inverse mode: estimate snow grain size from the measurement at a given wavelength
+    if invert_snow_grain_size == True:
+        alpha = -Ref0*np.log(invert_ref_obs/Ref0)/(Ks*Kv)
+        snow_grain_size = alpha*alpha*lmbd[invert_ilmbd]*1e-9/(4.*np.pi*kice[invert_ilmbd]*13)
+        return snow_grain_size
+
+    # - Normal mode: compute reflectance
+
+    #  if multi_dir:
+    RefS  = np.zeros((nlmbd,len(vza)), np.float)
 
     if nlmbd < len(vza):
         for i in range(nlmbd):
             RefS[i,:] = Ref0*np.exp(-alpha[i]*Ks*Kv/Ref0)
     else:
         for i in range(len(vza)):
-            RefS[:,i] = Ref0[i]*np.exp(-alpha*Ks*Kv[i]/Ref0[i])
+            RefS[:,i] = Ref0[i]*np.exp(-alpha*Ks[i]*Kv[i]/Ref0[i])
 
-    ans = RefS.reshape((nlmbd,len(vza),len(phi)))
 
-    if do_spectral_averaging == True: # computation for the individual wavebands
-        ans = np.zeros(len(vza_meshed), np.float)
-        for i in range(len(vza_meshed)):
-            ans[i] = np.average(RefS[:,i])
-        ans = RefS.reshape((1,len(vza),len(phi)))
+    ans = RefS    #.reshape((nlmbd,len(vza),len(phi)))
 
-    # return value
+
+    # -
+    # - return value
+    # -
     return  ans
 
-# END function_brdf_snow
+# END brdf_snow
 # ===================================================================
 
 
+
 # ===================================================================
 #  F1 function of the Li-Sparse Reciprocal BRDF model
 #
@@ -735,10 +740,6 @@ def brdf_land_f1(sza,vza,phi):
 
     dr = np.pi/180
 
-    #print sza.shape
-    #print vza.shape
-    #print phi.shape
-
     delta = np.sqrt(np.tan(sza*dr)**2 + np.tan(vza*dr)**2 - 2.0* np.tan(sza*dr)*np.tan(vza*dr)*np.cos(phi*dr))
     sec_sza = 1/np.cos(sza*dr)
     sec_vza = 1/np.cos(vza*dr)
@@ -817,7 +818,7 @@ def define_vza_hotspot(sza, vza):
 # ===================================================================
 # Define vza,phi domain for displaying polar plots
 #
-def define_angles_polar_plot( vza, step_polar):
+def define_angles_polar_plot( vza, step_polar ):
     ''' Define vza,phi domain for displaying polar plots.
 
      Inputs
@@ -830,12 +831,12 @@ def define_angles_polar_plot( vza, step_polar):
     '''
 
     vza_max = np.amax(vza)
-    vza_val = np.linspace(0,vza_max,step_polar)
-    phi_val = np.linspace(0,360,step_polar)
+    vza_val = np.linspace(0, vza_max, step_polar)
+    phi_val = np.linspace(0, 360, step_polar)
     vza_val,phi_val = np.meshgrid(vza_val,phi_val)
-    phi_meshed, vza_meshed = np.meshgrid(phi, vza) # CHANGE_CB
+    #phi_meshed, vza_meshed = np.meshgrid(phi, vza) # CHANGE_CB
 
-    return [vza_val.ravel(),phi_val.ravel()]
+    return vza_val.ravel(), phi_val.ravel()
 # END define_vza
 # ===================================================================
 
@@ -846,7 +847,8 @@ def define_angles_polar_plot( vza, step_polar):
 # for the determination of the BRDF as a function of the wind speed
 
 def ocean_specular_reflectance( ti, n):
-    ''' Computation of the specular reflectance over ocean / required
+    '''
+    Computation of the specular reflectance over ocean / required
     for the determination of the BRDF as a function of the wind speed
 
     Specular reflectance is determined thanks to Fresnel equations,
@@ -855,9 +857,11 @@ def ocean_specular_reflectance( ti, n):
 
     Inputs
         - incidence angle :       scalar or vector
-    - water refractive index:      scalar or vector
+        - water refractive index:      scalar or vector
+
     Outputs
         - reflectance [len(n)]
+
     '''
     dr = np.pi/180.
 
@@ -867,13 +871,13 @@ def ocean_specular_reflectance( ti, n):
     if type(ti) != type(np.array(0)): ti = [ti]
     ti = np.array(ti)
 
-    nwl = len(n)
+    nlmbd = len(n)
     nang = len(ti)
-    tv = np.zeros((nwl,nang))
-    rho_sp = np.zeros((nwl,nang))
+    tv = np.zeros((nlmbd,nang))
+    rho_sp = np.zeros((nlmbd, nang))
 
-    if nwl< nang:
-        for i in range(nwl):
+    if nlmbd< nang:
+        for i in range(nlmbd):
             tv = np.arcsin(np.sin(ti*dr)/n[i])
             ctv = np.cos(tv)
             Rs = (cti-n[i]*ctv)/(cti+n[i]*ctv)
