@@ -6,7 +6,7 @@
 #  NOVELTIS: Cedric Bacour / Ivan Price
 ###############################################################
 
-#from __init__ import *
+
 
 import numpy as np
 
@@ -89,7 +89,7 @@ def main(job, do_compute_error = False):
     err_reflectance_land = np.empty((ref_land.shape[0], len(job.cfg.lmbd)), np.float32)
     err_reflectance_land[:] = np.NaN
 
-    ##err_reflectance_land = np.array([])
+
 
 
     # - for each pixel in the land pixels collection
@@ -104,9 +104,8 @@ def main(job, do_compute_error = False):
             # the one in reflectance_land function may be good
 
             ans = reflectance_spectrum_land( ref_land[idx,:],
-                                                            job.cfg.lmbd,
-                                                            job.cfg
-                                                          )
+                                             job.cfg.lmbd,
+                                             job.cfg)
             reflectance[idx,:] = ans
 
 
@@ -127,16 +126,29 @@ def main(job, do_compute_error = False):
                                                                        )
             ##ipix = ipix+1
 
-    # - snow (PRELIMINARY VERSION / TO BE UBDATED)
+    # - snow: the snow model is fitted to the observations
     if len(data['idx_snow']) > 0:
 
+        # initialiaze the structure for saving informations on snow pixels
+        job.data['snow'] = {}
+
+        # in this first pass, we estimate the various components necessary for processing
+        # snow pixels
         for idx in data['idx_snow']:
+            job.data['snow'][idx] = {}
             ans = process_brdf.function_brdf_snow( job.cfg,
                                                    sza  = job.cfg.sza_std,
                                                    vza  = job.cfg.vza_std,
                                                    phi  = job.cfg.phi_std,
-                                                   lmbd = job.cfg.lmbd)
-            reflectance[idx,:] = ans.ravel()
+                                                   lmbd = job.cfg.lmbd,
+                                                   fit2obs = True,
+                                                   ref_obs = ref_land[idx,:])
+            reflectance[idx,:] = ans[0].ravel()
+
+            job.data['snow'][idx]['fveg'] = ans[1]['fveg']
+            job.data['snow'][idx]['snow_grain_size'] = ans[1]['snow_grain_size']
+            job.data['snow'][idx]['rveg'] = ans[1]['rveg']
+            job.data['snow'][idx]['rtot'] = ans[1]['rtot']
 
     # -
     # - Ocean processing
@@ -343,19 +355,19 @@ def reflectance_spectrum_land(data_in, lmbd, cfg, error = None):
         # ans = np.dot(Cland.ACP_eigenvectors,buf) + Cland.ACP_SpecMean
 
         # HERE IS ALL THE TIME FOR CALCULATION
-        ans = np.dot(cfg.land.ACP_mat,ref_out) + cfg.land.ACP_SpecMean
+        ans = np.dot(cfg.land.ACP_mat, ref_out) + cfg.land.ACP_SpecMean
         #
 
         # reflectance values must be > 0.005
-        mask_low = np.ma.masked_less(ans,0.005)
-        ans = np.array(np.ma.filled(mask_low,0.005))
+        mask_low = np.ma.masked_less(ans, 0.005)
+        ans = np.array(np.ma.filled(mask_low, 0.005))
 
 
     # Compute the error on the reflectance spectrum (return standard deviation)
     else:
         ans = np.zeros(lmbd.shape,np.float64)
         for i in range((lmbd.shape)[0]):
-            buf = np.dot(cfg.land.ACP_mat[i,:],np.array(data_in))
+            buf = np.dot(cfg.land.ACP_mat[i,:], np.array(data_in))
             ans[i] = np.sqrt(np.dot(buf,np.transpose(cfg.land.ACP_mat[i,:])))
 
 
