@@ -15,11 +15,11 @@ import tempfile
 import os
 import random
 import math
-from PIL import Image
+#from PIL import Image
 import calendar
 
 # matplotlib doesn't work without a writable home directory, as maybe the case when
-# run by the web server for the web site. If this is so then we assign a temp dir as 
+# run by the web server for the web site. If this is so then we assign a temp dir as
 # it's home directory to keep it happy
 home_dir = os.getenv('USERPROFILE') or os.getenv('HOME')
 if not os.path.exists(home_dir):
@@ -60,7 +60,7 @@ import adam
 # gestion of the missval pixels
 #
 # ===================================================================
-def map(data_map, lats, lons, enlarge = False, fig = None,
+def map_plt(data_map, lats, lons, enlarge = False, fig = None,
         save = False, save_format = 'png', path = None, filename = None,
         missval = None, fill_missval = 0):
 
@@ -162,7 +162,7 @@ def map_pil(adam_job, spectral_domain_index=0, width=250, output_dir=None):
     # get the data variable
     data = adam_job.data['reflectance_averaged']
 
-    
+
     data = np.ma.masked_equal(data, -9999)
     # if this is 3d, it means we were passed an array with different spectra, we should
     # map only the one we are asked to map
@@ -211,7 +211,7 @@ def map_pil(adam_job, spectral_domain_index=0, width=250, output_dir=None):
       3: {'min': 0, 'max': 1.41174},
       4: {'min': 0, 'max': 1.26724}
     }
-    
+
 
 
 
@@ -293,7 +293,7 @@ def map_raw_values(adam_job, spectral_domain_index=0, width=250, output_dir=None
 
     # get the data variable
     data = adam_job.data['reflectance_averaged']
-    
+
     #data = adam_job.data['reflectance_MODIS']
 
     # if this is 3d, it means we were passed an array with different spectra, we should
@@ -364,8 +364,8 @@ def map_raw_values(adam_job, spectral_domain_index=0, width=250, output_dir=None
 #      5: {'min': 0, 'max': 0.7 },
 #      6: {'min': 0, 'max': 0.7 }
 #    }
-    
-    
+
+
     this_range = range_dict[spectral_domain_index]['max'] - range_dict[spectral_domain_index]['min']
     #stretched = data.clip(range_dict[spectral_domain_index]['min'], range_dict[spectral_domain_index]['max'])
     #stretched = data * 400/range_dict[spectral_domain_index]['max']
@@ -385,7 +385,7 @@ def map_raw_values(adam_job, spectral_domain_index=0, width=250, output_dir=None
     output_driver = gdal.GetDriverByName( "GTiff" )
     output_dataset = output_driver.Create( filename + '.tif', int(width), int(height), 1, gdal.GDT_Float32 )
     #output_dataset = output_driver.Create( filename + '.tif', int(width), int(height), 1, gdal.GDT_Int16 )
-    
+
     #output_dataset = output_driver.Create( filename + '.tif', int(width), int(height), 1, gdal.GDT_Byte )
 
     #lon = adam_job.data['longitude']
@@ -419,7 +419,7 @@ def map_raw_values(adam_job, spectral_domain_index=0, width=250, output_dir=None
 # Input
 #  - data dictionnary containing mean, max, min, and std
 # ===================================================================
-def main_ref_spectra(job, case = None, indices=None, title = None, graph_error_indices=None):
+def main_ref_spectra(job, case = None, indices=None, title = None):
 
     cfg = job.cfg
     # -
@@ -431,15 +431,17 @@ def main_ref_spectra(job, case = None, indices=None, title = None, graph_error_i
             # if we were given an empty array, or the length of the data is for some reason zero
             # we just return
             return
-            
+
         # get the data for the pixel graph, store its original shape
         original_shape = job.data['BRDF'].shape
-        
+
         # if the BRDF variable is more than 2d (i.e. has lat,long and spectra),
         # we need to flatten the data to a 1d array of reflectances per wavelength
         if len(job.data['BRDF'].shape) > 2:
             job.data['BRDF'] = job.data['BRDF'].reshape(job.data['BRDF'].shape[0] * job.data['BRDF'].shape[1], len(cfg.lmbd))
 
+        if len(job.data['ref_land'].shape) > 2:
+            job.data['ref_land'] = job.data['ref_land'].reshape(job.data['ref_land'].shape[0] * job.data['ref_land'].shape[1], len(job.cfg.modis_wavebands))
 
         # if we weren't given the indices to graph, assume all
         if indices == None:
@@ -447,6 +449,7 @@ def main_ref_spectra(job, case = None, indices=None, title = None, graph_error_i
 
         # initialise the graphable data
         ref = np.zeros((len(cfg.lmbd), len(indices)))
+        ref_obs = np.zeros((len(job.cfg.modis_wavebands), len(indices)))
 
         legend = []
         # we need a set of coordinate pairs from the lat/longs,
@@ -454,9 +457,13 @@ def main_ref_spectra(job, case = None, indices=None, title = None, graph_error_i
         x,y = np.meshgrid(job.data['longitude'], job.data['latitude'])
 
         # cycle through each pixel
+
+
         for i in range(len(indices)):
             # load the graphable data
             ref[:,i] = np.ma.array(job.data['BRDF'][indices[i]])
+            ref_obs[:,i] = np.ma.array(job.data['ref_land'][indices[i]])
+
             # get the lat long using the same index in our meshgrid
             lat = y.ravel()[indices[i]]
             lon = x.ravel()[indices[i]]
@@ -466,14 +473,19 @@ def main_ref_spectra(job, case = None, indices=None, title = None, graph_error_i
 
             # build legend text
             legend.extend(['pix %i (%s,%s)'%(indices[i],txt_lat,txt_lon) ])
-        
+
+        # mask missval
+        # idxNAN = np.ma.where(ref_obs == cfg.missval_2b)
+        # ref_obs[idxNAN] = np.nan
 
         error_array = job.data['reflectance_err_land']
         if len(error_array.shape) > 2:
             error_array = error_array.reshape(error_array.shape[0] * error_array.shape[1], error_array.shape[2] )
         error_array = error_array[indices,:]
 
-        draw_ref_spectra(cfg.lmbd, ref, job, title=title, legend=legend, error_array=error_array)
+        draw_ref_spectra(cfg.lmbd, ref, job, title=title, legend=legend, error_array=error_array,
+                         lmbd_obs = job.cfg.modis_wavebands, ref_obs = ref_obs)
+
 
         job.data['BRDF'] = job.data['BRDF'].reshape(original_shape)
         job.data['reflectance_err_land'] = job.data['reflectance_err_land'].reshape(original_shape)
@@ -550,12 +562,16 @@ def main_ref_spectra(job, case = None, indices=None, title = None, graph_error_i
 # Input
 #  - data dictionnary containing mean, max, min, and std
 # ===================================================================
-def draw_ref_spectra(x, y, job, title=None, legend=None, yerr=None, error_array=None):
+def draw_ref_spectra(x, y, job, title=None, legend=None, yerr=None, error_array=None, lmbd_obs = None, ref_obs = None):
+
+    colors = ['b', 'g', 'r', 'c', 'm', 'y', 'k']
+    nel = y.shape[1]
 
     fig = plt.figure(figsize=(9,6))
     if title==None:
         title = ''
-    plt.plot(x,y)
+    # plt.plot(x,y)
+    for i in range(nel): plt.plot(x,y[:,i],colors[i])
 
     if error_array is not None:
         # error array was pixel, spect: we need spect, pixel
@@ -564,14 +580,17 @@ def draw_ref_spectra(x, y, job, title=None, legend=None, yerr=None, error_array=
         maximum = y + error_array
         plt.plot(x, minimum,'grey')
         plt.plot(x, maximum,'grey')
-        
-        
+
+
     plt.xlabel('wavelength (nm)')
     plt.ylabel('reflectance')
     plt.legend(legend, loc='upper right')
     plt.xlim(400,4000)
     plt.title(title)
 
+    # add MODIS observations
+    for i in range(nel):  plt.plot(lmbd_obs,ref_obs[:,i],marker='o',color=colors[i],linestyle='')
+
     # add the text explaination
     plt.subplots_adjust(left=0.1, right=0.8, top=0.95, bottom=0.1)
     annotation_x = 535
@@ -614,16 +633,16 @@ def main_brdf_time(job, case=None):
 
         data = job.time_series
         data = np.array(data)
-        print data.shape
+        #        print data.shape
         # reshape data from (12, x_pixels, y_pixels, 1) -> (12, total)
         data = data.reshape((data.shape[0], data.shape[1] * data.shape[2] ))
-        
+
 
         for surface in surfaces:
             title = 'BRDF Time Series - Pixels - %s' % surface
-            
+
             # we need a list of all the unique pixel indexes for this surface type that occur during the year
-            # in order to follow each relevant pixels throughout the year, regardless of if it disappears for 
+            # in order to follow each relevant pixels throughout the year, regardless of if it disappears for
             # certain months
             possible_indexes = []
             for month_number in range(12):
@@ -632,27 +651,26 @@ def main_brdf_time(job, case=None):
                     if not idx in possible_indexes:
                         possible_indexes.append(idx)
             possible_indexes = np.array(possible_indexes)
-            
+
             job.set_status('possible pixel indexes for surface %s (across all months) are: %s' % (surface, [possible_indexes]))
-            
+
             # if there is not a single pixel for any month we will skip this surface
             if len(possible_indexes) < 1:
                 continue
-            
-            
+
+
             #graph_data is the array to be graphed, it has shape 12 months x the max number of pixels for this surface type
             graph_data = np.empty((12, len(possible_indexes)))
             graph_data[:] = np.NaN
-            
+
             for month_index in range(12):
                 indices = job.time_series_idxs[month_index][surface]
-                job.set_status('for month %s adding data for indexes: %s' % (month_index, [indices]))
                 # for each pixel for this month/surface combo, fill the appropriate slot in the graph_data array
                 for idx in indices:
                     graph_data[month_index][possible_indexes == idx] = data[month_index][idx]
 
 
-                
+
 
             # init the x axis values 0 -> 11
             x = np.arange(12)
@@ -671,12 +689,22 @@ def main_brdf_time(job, case=None):
 
             # init the graph
             fig = plt.figure(figsize=(9,6))
+            plt.subplots_adjust(left=0.1, right=0.8, top=0.95, bottom=0.1)
             plt.clf()
 
             # plot the data
             plt.plot(x, y, 'o-')
 
             plt.xlabel('')
+            
+            locs, labels = plt.yticks()
+            # if the values are all low (as is the case for ocean pixels) then we
+            # present the y axis values in scientific notation
+            if locs[-1] < 0.1:
+                plt.subplots_adjust(left=0.15, right=0.8, top=0.95, bottom=0.1)
+                plt.yticks(locs, map(lambda x: "%.4e" % x, locs))
+                
+                
             plt.ylabel('Reflectance')
             # legend
             leg = plt.legend(legend, loc='upper right')
@@ -688,13 +716,13 @@ def main_brdf_time(job, case=None):
             plt.title(title)
 
             # add the text explaination
-            plt.subplots_adjust(left=0.1, right=0.8, top=0.95, bottom=0.1)
+            
             annotation_x = 535
             annotation_y = 390
             plt.annotate('ADAM', xy=(annotation_x, annotation_y), fontsize=22, xycoords='figure pixels')
             annotation_y -= 20
             plt.annotate('Reflectance Database', xy=(annotation_x, annotation_y), fontsize=10, xycoords='figure pixels')
-            annotation_y -= 150
+            annotation_y -= 160
 
             annotation = 'Month: %s\n' % job.get_month_name()
             annotation += 'Lats:\n   %.03f:%.03f\n' % (job.data['latitude'].min(), job.data['latitude'].max())
@@ -733,13 +761,23 @@ def main_brdf_time(job, case=None):
             # init the graph
             fig = plt.figure(figsize=(9,6))
             plt.clf()
+            plt.subplots_adjust(left=0.1, right=0.8, top=0.95, bottom=0.1)
             plt.errorbar(x, data[surface]['mean'], yerr = data[surface]['std'], color = '0.8')
             plt.plot(x, data[surface]['mean'],'ro-', label = 'mean')
             plt.plot(x, data[surface]['min'], 'go-', label = 'min')
             plt.plot(x, data[surface]['max'], 'bo-', label = 'max')
             plt.xlabel('')
             plt.xticks( np.arange(12), calendar.month_abbr[1:13])
-            plt.ylabel('reflectance')
+            
+            
+            locs, labels = plt.yticks()
+            # if the values are all low (as is the case for ocean pixels) then we
+            # present the y axis values in scientific notation
+            if locs[-1] < 0.1:
+                plt.subplots_adjust(left=0.15, right=0.8, top=0.95, bottom=0.1)
+                plt.yticks(locs, map(lambda x: "%.4e" % x, locs))
+
+            plt.ylabel('Reflectance')
             # legend
             loc='upper left'
             if job.sza < 0:
@@ -754,13 +792,13 @@ def main_brdf_time(job, case=None):
             plt.title('Time Series - %s pixels (max %s), Extent: %s' % (surface, data[surface]['npts'], extent))
 
             # add the text explaination
-            plt.subplots_adjust(left=0.1, right=0.8, top=0.95, bottom=0.1)
+            
             annotation_x = 535
             annotation_y = 390
             plt.annotate('ADAM', xy=(annotation_x, annotation_y), fontsize=22, xycoords='figure pixels')
             annotation_y -= 20
             plt.annotate('Reflectance Database', xy=(annotation_x, annotation_y), fontsize=10, xycoords='figure pixels')
-            annotation_y -= 150
+            annotation_y -= 160
 
             annotation = 'Month: %s\n' % job.get_month_name()
             annotation += 'Lats:\n   %.03f:%.03f\n' % (job.data['latitude'].min(), job.data['latitude'].max())
@@ -773,7 +811,7 @@ def main_brdf_time(job, case=None):
 
 
             # save the graph
-            filename = job.build_filename('BRDFplane_%s' % surface, 'png')
+            filename = job.build_filename('BRDFtime_%s' % surface, 'png')
             plt.savefig(filename, dpi=72, facecolor='None', edgecolor='None',
                         orientation='portrait', papertype=None, format= 'png', bbox_inches=None)
             plt.close(fig)
@@ -809,8 +847,8 @@ def main_brdf_principal_plane(job, graph_error=False, case=None, indices=None, t
             # if we were given an empty array, or the length of the data is for some reason zero
             # we just return
             return
-        
-        
+
+
         data = job.data['BRDF'][:,:, spectral_domain_index,:, phi_index]
         #data = job.data['BRDF']
         data_err = job.data['BRDF_err_land'][:,:, spectral_domain_index,:, phi_index]
@@ -829,7 +867,7 @@ def main_brdf_principal_plane(job, graph_error=False, case=None, indices=None, t
         # initialise the graphable data
         brdf = np.zeros((vza.shape[0],len(indices)))
         brdf_err = np.zeros((vza.shape[0],len(indices)))
-        
+
         legend = []
         # cycle through each pixel
         for i in range(len(indices)):
@@ -862,9 +900,11 @@ def main_brdf_principal_plane(job, graph_error=False, case=None, indices=None, t
         for surface in surfaces:
             fig = plt.figure(figsize=(9,6))
 
-            # ivan changed this
-            # if this is none it means no ocean or land pixels were found
+            print surface
             if data[surface] != None:
+                print  data[surface]['mean'].shape
+                print  data[surface]['std'].shape
+                print vza.shape
                 plt.errorbar(vza, data[surface]['mean'], yerr = data[surface]['std'],color = '0.8')
                 plt.plot(vza, data[surface]['mean'],'r', label = 'mean')
                 plt.plot(vza, data[surface]['min'],'g', label = 'min')
@@ -932,7 +972,7 @@ def draw_brdf_principal_plane(x, y, job, error_array=None, title=None, legend=No
     fig = plt.figure(figsize=(9,6))
     plt.clf()
     lplt = plt.plot(x,y)
-    
+
 
     if error_array is not None:
 
@@ -945,16 +985,16 @@ def draw_brdf_principal_plane(x, y, job, error_array=None, title=None, legend=No
 
         plt.plot(x, minimum,'grey')
         plt.plot(x, maximum,'grey')
-    
+
     xl  = plt.xlabel('view zenith angle (deg)')
     yl  = plt.ylabel('reflectance')
-    
+
     # try to detect where the peak is to put the legend on the other side
     if np.argmax(y) < 25:
         leg = plt.legend(legend, loc='upper right')
     else:
         leg = plt.legend(legend, loc='upper left')
-        
+
     xlim = plt.xlim(x.min(),x.max())
     ltitle = plt.title(title)
 
@@ -999,19 +1039,32 @@ def draw_brdf_principal_plane(x, y, job, error_array=None, title=None, legend=No
 # TODO : add a colorbar
 #
 # ===================================================================
-def main_brdf_polar_plot(job, vza, phi, case = 'pixels', indices = None, title=None, three_d=False, waveband_index=0):
-
-
+def main_brdf_polar_plot(job, case = 'pixels', indices = None, title=None, three_d=False, waveband_index=0):
+    ''' Draw a polar plot of brdf data. 
+    
+    handles the meshing of vza and phi values internally.
+    
+    calls draw_3d_plot or draw_polar_plot to do the actual work.
+    
+    '''
+    
+    # we need vza and phi to be of equal shapes, so we mesh them here.
+    phi_meshed, vza_meshed = np.meshgrid(job.phi, job.vza)
+    phi_meshed = phi_meshed.ravel()
+    vza_meshed = vza_meshed.ravel()
+        
     # -
     # - Processing for individual pixels
     # -
     if case == 'pixels':
+        #data = job.data['BRDF'][:,:, spectral_domain_index,:, phi_index]
+        # data shape is long x lat x wavebands x vza x phi
+        # we select only one wave band, and flatten the long x, lat y dimensions
+        data = job.data['BRDF'][:,:, waveband_index, :,:]
+        data = data.reshape(data.shape[0] * data.shape[1], data.shape[2], data.shape[3])
+        # data shape is now pixel_index x vza x phi
 
-        # normally data shape is long x lat x wavebands x vza
-        # we want an array or single pixels each with the vza array
-        data = job.data['BRDF'][:,:, waveband_index, :]
-        data = data.reshape(data.shape[0] * data.shape[1], data.shape[2])
-
+         
         # if we weren't given the indices to graph, assume all
         if indices == None:
             indices = range(len(data))
@@ -1025,8 +1078,9 @@ def main_brdf_polar_plot(job, vza, phi, case = 'pixels', indices = None, title=N
         x,y = np.meshgrid(job.data['longitude'], job.data['latitude'])
 
         # initialise the graphable data
-        brdf = np.zeros((vza.shape[0],len(indices)))
+        brdf = np.zeros((vza_meshed.shape[0], len(indices)))
 
+        
         legend = []
         # cycle through each pixel
         for i in range(len(indices)):
@@ -1043,10 +1097,12 @@ def main_brdf_polar_plot(job, vza, phi, case = 'pixels', indices = None, title=N
 
             if (three_d):
                 filename = job.build_filename('BRDF_3D', 'png', lats=[lat], lons=[lon])
-                draw_3d_plot(data[indices[i]], vza, phi, job, label=label, filename=filename, title='%s - %s' % (title, i))
+                draw_3d_plot(data[indices[i]], vza_meshed, phi_meshed, job, label=label,
+                             filename=filename, title='%s - %s' % (title, i))
             else:
                 filename = job.build_filename('BRDF_polar', 'png', lats=[lat], lons=[lon])
-                draw_polar_plot(data[indices[i]], vza, phi, job, label=label, filename=filename, title='%s - %s' % (title, i))
+                draw_polar_plot(data[indices[i]], vza_meshed, phi_meshed, job, label=label,
+                                filename=filename, title='%s - %s' % (title, i))
 
 
 
@@ -1055,39 +1111,54 @@ def main_brdf_polar_plot(job, vza, phi, case = 'pixels', indices = None, title=N
     # - Processing statistics for each surface type
     # -
     if case == 'stats':
-
+        data = job.data['stats']
         surfaces = data.keys()
 
-        var2draw = ['mean','min','max']
-        ifig = 0
-        for i in range(len(surfaces)):
-            surf = surfaces[i]
+        vars2draw = ['mean','min','max']
+        title_string = title
+        for surf in surfaces:
 
-            if type(data[surf]) == type({}):
-
-                for j in range(len(var2draw)):
-                    brdf = data[surf][var2draw[j]]
-                    draw_polar_plot(brdf, vza, phi, job, title = var2draw[j]+' BRDF for '+surf+'pixels (%i)'% (data[surf]['npts']) )
-
-                    ifig = ifig+1
+            for var in vars2draw:
+                if data[surf] is None: continue
+                brdf = data[surf][var]
+                title = '%s %s: %s (%i pixels)' % (surf.title(), title_string, var.title(), data[surf]['npts'])
+                # the stats function gives us flattened brdf data, we need to reshape that to 
+                # shape: vza, phi
+                brdf = np.reshape(brdf, (len(job.vza), len(job.phi)))
+                
+                if (three_d):
+                    filename = job.build_filename('BRDF_Stats_3D', 'png')
+                    draw_3d_plot(brdf, vza_meshed, phi_meshed, job, filename=filename, title=title)
+                else:
+                    filename = job.build_filename('BRDF_Stats', 'png')
+                    draw_polar_plot(brdf, vza_meshed, phi_meshed, job, title=title)
+                
 
 
 
 
 
 
-def draw_3d_plot(brdf, vza, phi, job, title=None, label=None, filename=None):
 
-
-    # - reshape brdf
-    sz = brdf.shape[0]
-    vza = np.reshape(vza,  (np.sqrt(sz),np.sqrt(sz))  )
-    phi = np.reshape(phi,  (np.sqrt(sz),np.sqrt(sz))  )
-    brdf = np.reshape(brdf, (np.sqrt(sz),np.sqrt(sz)) )
+def draw_3d_plot(brdf, vza_meshed, phi_meshed, job, title=None, filename=None, label=None):
+    ''' Performs the actual graph for a 3d polar plot.
+    
+    importantly vza_meshed and phi_meshed must be the same shape, 
+    for example created by meshgrid like this:
+        
+        phi_meshed, vza_meshed = np.meshgrid(job.phi, job.vza)
+        phi_meshed = phi_meshed.ravel()
+        vza_meshed = vza_meshed.ravel()
+        
+    '''
+    
+    # we need vza and phi to be in the same shape as the brdf array (len(vza) x len(phi))
+    vza_meshed = np.reshape( vza_meshed,  brdf.shape )
+    phi_meshed = np.reshape( phi_meshed,  brdf.shape )
 
     # - Polar coordinates
-    x = vza*np.cos(phi*np.pi/180)
-    y = vza*np.sin(phi*np.pi/180)
+    x = vza_meshed*np.cos(phi_meshed*np.pi/180)
+    y = vza_meshed*np.sin(phi_meshed*np.pi/180)
 
     fig = plt.figure(figsize=(9,6))
 
@@ -1099,8 +1170,9 @@ def draw_3d_plot(brdf, vza, phi, job, title=None, label=None, filename=None):
 
     ax.set_zlabel('Reflectance')
     # this only works for matplotlib version > 1
-    ax.set_title('%s : %s' % (title, label))
-
+    ax.set_title(title)
+    if label is not None: ax.set_title('%s: (%s)' % (title, label))
+    
     # rotate the view to make it look like the polar graph
     # ax.view_init(elev, azim)
     ax.view_init(30, 5)
@@ -1144,26 +1216,31 @@ def draw_3d_plot(brdf, vza, phi, job, title=None, label=None, filename=None):
 # TODO : add a colorbar
 #
 # ===================================================================
-def draw_polar_plot(brdf, vza, phi, job, title=None, label=None, filename=None):
-
+def draw_polar_plot(brdf, vza_meshed, phi_meshed, job, title=None, filename=None, label=None):
+    ''' Performs the actual graph for a 2d polar plot.
+    
+    importantly vza_meshed and phi_meshed must be the same shape, 
+    for example created by meshgrid like this:
+        
+        phi_meshed, vza_meshed = np.meshgrid(job.phi, job.vza)
+        phi_meshed = phi_meshed.ravel()
+        vza_meshed = vza_meshed.ravel()
+        
+    '''
 
     fig = plt.figure(figsize=(9,6))
 
-    # - reshape brdf
-    sz = brdf.shape[0]
-
-    vza = np.reshape(vza,(np.sqrt(sz),np.sqrt(sz)))
-    phi = np.reshape(phi,(np.sqrt(sz),np.sqrt(sz)))
-    brdf = np.reshape(brdf,(np.sqrt(sz),np.sqrt(sz)))
-
+    vza_meshed = np.reshape( vza_meshed,  brdf.shape )
+    phi_meshed = np.reshape( phi_meshed,  brdf.shape )
+    
     # -
     # - Draw contourf
     # -
 
     # - Polar coordinates
-    maxrho = vza.max()
-    x = vza*np.cos(phi*np.pi/180)
-    y = vza*np.sin(phi*np.pi/180)
+    maxrho = vza_meshed.max()
+    x = vza_meshed*np.cos(phi_meshed*np.pi/180)
+    y = vza_meshed*np.sin(phi_meshed*np.pi/180)
 
     # - draw surface
     ax = plt.subplot(111)
@@ -1218,8 +1295,8 @@ def draw_polar_plot(brdf, vza, phi, job, title=None, label=None, filename=None):
 
     # - Add colorbar, 'fraction' positions it left and right
     plt.colorbar(id_cont, fraction=0.3)
-    plt.title('%s : %s' % (title, label))
-
+    plt.title(title)
+    if label is not None: ax.set_title('%s: (%s)' % (title, label))
 
     plt.subplots_adjust(left=0.05, right=0.6, top=0.90, bottom=0.05 )
     annotation_x = 535
