@@ -1,12 +1,10 @@
 # -*- coding: utf-8 -*-
 """
 title           : brdf_graph.py
-description     : script using the ADAM Toolkit demonstating taking making BRDF graphs
-author          : Ivan.Price@noveltis.fr
-date            : Tue Jul 31 16:55:26 2012
-version         : 0.1
+description     : script using the ADAM Toolkit demonstating making making BRDF graphs
+version         : 1.0
 usage           : python brdf_graph.py
-notes           :
+notes           : 
 """
 
 
@@ -23,15 +21,14 @@ if not '../' in sys.path:
 
 # import the adam libraries
 import adam
-import adam_config as adam_config
-
-
-# my_path_data       = 'p://3895-ADAM/code/data/'
-# my_output_root_dir = 'Y://web-root/adam/web-output/'
+import adam_config
 
 #my_path_data       = '/home/satellites9/cbacour/ADAM/Data/V1/'
 #my_output_root_dir = '/home/satellites9/cbacour/ADAM/OUTPUT/'
 
+# my_path_data       = 'p://3895-ADAM/code/data/'
+# my_output_root_dir = 'Y://web-root/adam/web-output/'
+
 # get an instance of an adam 'config' object, this will be attached to any jobs we run
 #my_config = adam_config.adam_config(path_data = my_path_data, output_root_dir = my_output_root_dir)
 my_config = adam_config.adam_config()
@@ -42,27 +39,28 @@ my_config = adam_config.adam_config()
 # the 'request_dictionary' from the logfile into this script
 request_dict = { 'fieldOperationType': 'brdf',
                  'fieldSpectralDomain': '850-860 nm',
-                 'fieldBRType': 'polar',
+                 'fieldBRType': 'plane',
                  'fieldSunZenith': 45,
                  'fieldViewZenith': 0,
                  'fieldRelAzimuth': 0,
                  'fieldMonth': 'jan',
 
-#                 'fieldCorner1Lat'   : 37.1,
-#                 'fieldCorner1Lon'   : -3.6,
-#                 'fieldCorner2Lat'   : 36.3,
-#                 'fieldCorner2Lon'   : -3.5
+                # snow with low reflectance values
+#                 'fieldCorner1Lat'   : 63.2,
+#                 'fieldCorner1Lon'   : 27.2,
+#                 'fieldCorner2Lat'   : 63.3,
+#                 'fieldCorner2Lon'   : 27.4
 
-                'fieldCorner1Lon': '-4.1',
-                'fieldCorner2Lon': '-2.6',
-                'fieldCorner2Lat': '37.5',
-                'fieldCorner1Lat': '36.8'
-                
-                # snow
-#                'fieldCorner1Lat'   : 63.2,
-#                'fieldCorner1Lon'   : 27.2,
-#                'fieldCorner2Lat'   : 63.3,
-#                'fieldCorner2Lon'   : 27.4
+                 'fieldCorner1Lat'   : 43.2,
+                 'fieldCorner1Lon'   : 0.5,
+                 'fieldCorner2Lat'   : 43,
+                 'fieldCorner2Lon'   : 0.6
+                 
+                 # snow with high reflectance values
+                # 'fieldCorner1Lon': '-47.2',
+                # 'fieldCorner2Lon': '-47.3',
+                # 'fieldCorner2Lat': '66.9',
+                # 'fieldCorner1Lat': '66.6'
                 }
 
 # initialise the job, providing us with a blank object ('job') to work with
@@ -92,24 +90,22 @@ job.process_reflectance()
 print 'define_vza_hotspot...'
 job.define_vza_hotspot()
 
-job.phi = np.linspace(0,100)
-
 print 'Perform BRDF calculation...'
 # populates job.data['BRDF'] and job.data['BRDF_err_land']
 job.process_brdf(do_spectral_averaging=True)
 
 
-# graphs are created differently depending on how meny pixels are to be rendered
+
+# graphs are created differently depending on how many pixels are to be rendered
 # if there are only a few pixels we generate graphs depicting individual pixels
 if job.is_pixel_request():
     print 'generating pixel graphs'
-    #    job.graph_principal_plane(case='pixels', indices=job.data['idx_land'],  title='BRDF Land pixel statistics', graph_error=True)
-    #job.graph_principal_plane(case='pixels', indices=job.data['idx_ocean'], title='BRDF Ocean pixel statistics')
+    job.graph_principal_plane(case='pixels', indices=job.data['idx_land'],  title='BRDF Land pixel statistics', graph_error=True)
+    job.graph_principal_plane(case='pixels', indices=job.data['idx_ocean'], title='BRDF Ocean pixel statistics')
     job.graph_principal_plane(case='pixels', indices=job.data['idx_snow'],  title='BRDF Snow pixel statistics' )
 
-    #job.graph_polar_plot(job.vza,job.phi, case='pixels', indices=job.data['idx_snow'], title='BRDF Ocean pixel statistics')
 
-    
+
 else:
     # or if there are many pixels we take the mean/max/min and graph only those stats
     print 'generating stats graphs'
@@ -119,4 +115,4 @@ else:
 
 
 job.save_netcdf()
-
+print 'job output was written to: %s' % job.job_output_dir
